import subprocess


def get_remote_name_of_url(git_remote_verbose_output, url_wanted):
    for line in git_remote_verbose_output.split("\n"):
        line_bits = [i.strip() for i in line.split() if i.strip()]
        if line_bits and line_bits[1] == url_wanted:
            return line_bits[0]


class Task:
    def __init__(
        self,
        directory: str,
        remote_user: str,
        remote_host: str,
        remote_port: str,
        app_name: str,
    ):
        self.directory = directory
        self.remote_user = remote_user
        self.remote_host = remote_host
        self.remote_port = remote_port
        self.app_name = app_name

    def _dokku_command(self, command):
        full_command = [
            "ssh",
            "-p" + self.remote_port,
            self.remote_user + "@" + self.remote_host,
        ]
        full_command.extend(command)
        process = subprocess.Popen(
            full_command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=self.directory,
        )
        stdout, stderr = process.communicate()
        return stdout.decode("utf-8"), stderr.decode("utf-8")

    def _get_git_remote_name(self):
        print("Configure git remote ...")
        git_remote_url: str = (
            "ssh://"
            + self.remote_user
            + "@"
            + self.remote_host
            + ":"
            + self.remote_port
            + "/"
            + self.app_name
        )

        process = subprocess.Popen(
            ["git", "remote", "-v"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=self.directory,
        )
        stdout, stderr = process.communicate()

        # TODO check stderr

        git_remote_name = get_remote_name_of_url(stdout.decode("utf-8"), git_remote_url)
        if not git_remote_name:
            # TODO find unique git remote name
            git_remote_name = "dokku"
            process = subprocess.Popen(
                ["git", "remote", "add", git_remote_name, git_remote_url],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                cwd=self.directory,
            )
            stdout, stderr = process.communicate()
        return git_remote_name



from dokkusd.config_models import VolumeConfigModel


def test_string_1():
    c = VolumeConfigModel(config="/storage", app_name="app_name")
    assert c.ensure_command == ["storage:ensure-directory", "app_name"]
    assert c.mount_command == [
        "storage:mount",
        "app_name",
        "/var/lib/dokku/data/storage/app_name:/storage",
    ]


def test_dict_1():
    c = VolumeConfigModel(
        config={"host_subdir": "database", "container_path": "/database"},
        app_name="app_name",
    )
    assert c.ensure_command == ["storage:ensure-directory", "app_name_database"]
    assert c.mount_command == [
        "storage:mount",
        "app_name",
        "/var/lib/dokku/data/storage/app_name_database:/database",
    ]



import json
import os
import subprocess

from .config_models import (
    EnvironmentVariableConfigModel,
    ServiceConfigModel,
    VolumeConfigModel,
)
from .util import Task


class Deploy(Task):
    def __init__(
        self,
        directory: str,
        remote_user: str,
        remote_host: str,
        remote_port: str,
        app_name: str,
        http_auth_user: str = None,
        http_auth_password: str = None,
        environment_variables_json_string: str = None,
        environment_variables: dict = {},
    ):
        super().__init__(
            directory=directory,
            remote_user=remote_user,
            remote_host=remote_host,
            remote_port=remote_port,
            app_name=app_name,
        )
        self.http_auth_user = http_auth_user
        self.http_auth_password = http_auth_password
        self._environment_variables: dict = environment_variables
        self.environment_variables_json_string = environment_variables_json_string

    def go(self) -> None:

        # --------------------- app.json
        app_json_name = os.path.join(self.directory, "app.json")
        app_json = {}
        if os.path.exists(app_json_name):
            with open(app_json_name) as fp:
                app_json = json.load(fp)

        # --------------------- git remote
        print("Configure git remote ...")
        git_remote_name = self._get_git_remote_name()

        # --------------------- Create app
        print("Create app ...")
        stdout, stderr = self._dokku_command(["apps:create", self.app_name])
        print(stdout)
        print(stderr)

        # --------------------- Services
        print("Configure services ...")
        services = app_json.get("dokkusd", {}).get("services", [])
        for service in services:
            service_model = ServiceConfigModel(service, self.app_name)
            stdout, stderr = self._dokku_command(service_model.create_command)
            print(stdout)
            print(stderr)
            stdout, stderr = self._dokku_command(service_model.link_command)
            print(stdout)
            print(stderr)

        # --------------------- Volumes
        print("Configure volumes ...")
        volumes = app_json.get("dokkusd", {}).get("volumes", [])
        for volume in volumes:
            volume_model = VolumeConfigModel(volume, self.app_name)
            stdout, stderr = self._dokku_command(volume_model.ensure_command)
            print(stdout)
            print(stderr)
            stdout, stderr = self._dokku_command(volume_model.mount_command)
            print(stdout)
            print(stderr)

        # --------------------- Env Vars
        print("Configure Environment Variables ...")
        envvars = app_json.get("dokkusd", {}).get("environment_variables", {})
        envvars.update(self._environment_variables)
        if self.environment_variables_json_string:
            environment_variables_dict = json.loads(
                self.environment_variables_json_string
            )
            envvars.update(environment_variables_dict)
        for key, value in envvars.items():
            environment_variable = EnvironmentVariableConfigModel(
                key, value, self.app_name
            )
            stdout, stderr = self._dokku_command(environment_variable.set_command)
            print(stdout)
            print(stderr)

        # --------------------- HTTP Auth
        if self.http_auth_user and self.http_auth_password:
            print("HTTP Auth ...")
            stdout, stderr = self._dokku_command(
                [
                    "http-auth:enable",
                    self.app_name,
                    self.http_auth_user,
                    self.http_auth_password,
                ]
            )
            print(stdout)
            print(stderr)

        # --------------------- Keep Git Dir
        if "keep_git_dir" in app_json.get("dokkusd", {}):
            print("Keep Git Dir ...")
            stdout, stderr = self._dokku_command(
                [
                    "git:set",
                    self.app_name,
                    "keep-git-dir",
                    "true" if app_json["dokkusd"]["keep_git_dir"] else "false",
                ]
            )
            print(stdout)
            print(stderr)

        # --------------------- Nginx
        if "nginx" in app_json.get("dokkusd", {}):
            nginx = app_json.get("dokkusd", {}).get("nginx")
            if isinstance(nginx, dict):
                if "client_max_body_size" in nginx:
                    print("Nginx: client-max-body-size ...")
                    stdout, stderr = self._dokku_command(
                        [
                            "nginx:set",
                            self.app_name,
                            "client-max-body-size",
                            str(nginx.get("client_max_body_size")),
                        ]
                    )
                    print(stdout)
                    print(stderr)

        # --------------------- Deploy
        print("Deploy ...")
        process = subprocess.Popen(
            ["git", "push", "-f", git_remote_name],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=self.directory,
        )
        stdout, stderr = process.communicate()
        print(stdout.decode("utf-8"))
        print(stderr.decode("utf-8"))



import argparse
import os
import sys

from .deploy import Deploy
from .destroy import Destroy


def main() -> None:
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(dest="subparser_name")

    ### Deploy
    deploy_parser = subparsers.add_parser("deploy")

    # host
    deploy_parser.add_argument(
        "--remotehost", help="Remote URL", default=os.getenv("DOKKUSD_REMOTE_HOST")
    )
    deploy_parser.add_argument(
        "--remoteport",
        help="Remote Port",
        default=os.getenv("DOKKUSD_REMOTE_PORT") or "22",
    )
    deploy_parser.add_argument(
        "--remoteuser",
        help="Remote User",
        default=os.getenv("DOKKUSD_REMOTE_USER") or "dokku",
    )

    # app details
    deploy_parser.add_argument(
        "--appname", help="App name", default=os.getenv("DOKKUSD_APP_NAME")
    )
    deploy_parser.add_argument(
        "--httpauthuser",
        help="HTTP Auth User",
        default=os.getenv("DOKKUSD_HTTP_AUTH_USER"),
    )
    deploy_parser.add_argument(
        "--httpauthpassword",
        help="HTTP Auth Password",
        default=os.getenv("DOKKUSD_HTTP_AUTH_PASSWORD"),
    )
    deploy_parser.add_argument(
        "--environmentvariablesjson",
        help="Environment Variables in JSON dictionary",
        default=os.getenv("DOKKUSD_ENVIRONMENT_VARIABLES_JSON"),
    )
    deploy_parser.add_argument(
        "--environmentvariablesprefixedby",
        help="Any Environmental variables prefixed with this will be given to the Dokku app.",
    )

    ### Destroy
    destroy_parser = subparsers.add_parser("destroy")
    # host
    destroy_parser.add_argument(
        "--remotehost", help="Remote URL", default=os.getenv("DOKKUSD_REMOTE_HOST")
    )
    destroy_parser.add_argument(
        "--remoteport",
        help="Remote Port",
        default=os.getenv("DOKKUSD_REMOTE_PORT") or "22",
    )
    destroy_parser.add_argument(
        "--remoteuser",
        help="Remote User",
        default=os.getenv("DOKKUSD_REMOTE_USER") or "dokku",
    )

    # app details
    destroy_parser.add_argument(
        "--appname", help="App name", default=os.getenv("DOKKUSD_APP_NAME")
    )

    ### Go
    args = parser.parse_args()

    if args.subparser_name == "deploy":

        if not args.remotehost:
            print("Must specify remote host!")
            sys.exit(-1)

        if not args.appname:
            print("Must specify app name!")
            sys.exit(-1)

        env_vars = {}
        if args.environmentvariablesprefixedby:
            for name, value in os.environ.items():
                if name.startswith(args.environmentvariablesprefixedby):
                    env_vars[name[len(args.environmentvariablesprefixedby) :]] = value

        deploy = Deploy(
            directory=os.getcwd(),
            remote_user=args.remoteuser,
            remote_host=args.remotehost,
            remote_port=args.remoteport,
            app_name=args.appname,
            http_auth_user=args.httpauthuser,
            http_auth_password=args.httpauthpassword,
            environment_variables_json_string=args.environmentvariablesjson,
            environment_variables=env_vars,
        )
        deploy.go()

    elif args.subparser_name == "destroy":

        if not args.remotehost:
            print("Must specify remote host!")
            sys.exit(-1)

        if not args.appname:
            print("Must specify app name!")
            sys.exit(-1)

        destroy = Destroy(
            directory=os.getcwd(),
            remote_user=args.remoteuser,
            remote_host=args.remotehost,
            remote_port=args.remoteport,
            app_name=args.appname,
        )
        destroy.go()


if __name__ == "__main__":
    main()



from .util import Task


class Destroy(Task):
    def go(self) -> None:

        # --------------------- Destroy app
        print("Destroy app ...")
        stdout, stderr = self._dokku_command(["dokkusd:destroy-cascade", self.app_name])
        print(stdout)
        print(stderr)



from shlex import quote


class ServiceConfigModel:
    def __init__(self, config, app_name):
        if isinstance(config, str):
            self.create_command = [config + ":create", app_name]
            self.link_command = [config + ":link", app_name, app_name]


class VolumeConfigModel:
    def __init__(self, config, app_name):
        if isinstance(config, str):
            self.ensure_command = ["storage:ensure-directory", app_name]
            self.mount_command = [
                "storage:mount",
                app_name,
                "/var/lib/dokku/data/storage/" + app_name + ":" + config,
            ]
        elif isinstance(config, dict):
            storage_sub_dir_name = app_name + "_" + config.get("host_subdir")
            container_path = config.get("container_path")
            self.ensure_command = ["storage:ensure-directory", storage_sub_dir_name]
            self.mount_command = [
                "storage:mount",
                app_name,
                "/var/lib/dokku/data/storage/"
                + storage_sub_dir_name
                + ":"
                + container_path,
            ]


class EnvironmentVariableConfigModel:
    def __init__(self, key, value, app_name):
        self.set_command = [
            "config:set",
            "--no-restart",
            app_name,
            key + "=" + quote(value),
        ]



import setuptools

setuptools.setup(
    name="DokkuSD",
    version="0.4.0",
    description="DokkuSD",
    long_description="DokkuSD",
    url="https://github.com/OpenDataServices/dokkusd-client",
    project_urls={
        "Documentation": "https://dokkusd-client.readthedocs.io/en/latest/",
        "Issues": "https://github.com/OpenDataServices/dokkusd-client/issues",
        "Source": "https://github.com/OpenDataServices/dokkusd-client",
    },
    author="Open Data Services",
    author_email="code@opendataservices.coop",
    license="MIT",
    packages=setuptools.find_packages(exclude=["test"]),
    classifiers=[],
    install_requires=[],
    extras_require={
        "Dev": [
            "sphinx",
            "black==22.8.0",
            "isort==5.10.1",
            "flake8==5.0.4",
            "mypy==0.981",
            "pytest",
        ]
    },
    python_requires=">=3.7",
)
